/**
 * @fileOverview
 * @name admin_system.js
 * @author Thomas Del Grande (based on packages.js by Roberto Oppermann Cordoni)
 */

/**
 * @typedef {Object} FwController~FileUploaderProperties - Configuration properties for the FileUploader module.
 * @property {Number} queueLimit -  Maximum count of files.
 * @property {boolean} removeAfterUpload - Remove files from the queue after uploading.
 */

/**
 * @class FwController
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @param {Object} $FileUploader - AngularJS module for upload files.
 *
 * @property {String} version - Current installed firmware version.
 * @property {boolean} loading_upload - True if an upgrade is being applied.
 * @property {FwController~FileUploaderProperties} uploader - Object for managing the upload of files.
 */

angular.module('ag561app').controller('FwController', ['$http', '$log','FileUploader','$timeout',
                                   function($http, $log, FileUploader, $timeout) {
  var ctrl = this;

  ctrl.upgrading = false;
  ctrl.loading = false;

  ctrl.clear_alert_messages = function() {
    ctrl.alert_upgrade = false;
    ctrl.alert_factoryreset = false;
  };
  ctrl.clear_alert_messages();

  /* FileUploader */
  ctrl.upload_filename = '';
  ctrl.uploader = new FileUploader({"url": '/api/upload'});
  ctrl.uploader.queueLimit = 1;
  ctrl.uploader.removeAfterUpload = true;

  ctrl.uploader.onCompleteItem = function(fileItem, response, status, headers) {
    /*
     * FileUploader Callback.
     * It is activated when the upload of an item is completed.
     */
    if (status == 201) {
      // If success on uploading file, upgrade firmware.
      ctrl.firmware_upgrade();
    }
  };

  ctrl.uploader.onAfterAddingFile = function(item) {
    ctrl.upload_filename = item._file.name;
    ctrl.alert_upgrade = false;
  };

  /**
   * Call the FileUploader object to upload file.
   * @function apply_upgrade
   * @memberof FwController
   */
  ctrl.apply_upgrade = function() {
    ctrl.upgrading = true;
    ctrl.uploader.uploadAll();
  };

  /**
   * Get current firmware (bundle) version that is installed in the system.
   * @function get
   * @memberof FwController
   */
  ctrl.get = function() {
    $http({
      method: 'GET',
      url: '/api/system/info',
    }).then(function(response) {
      // First function handles success
      ctrl.serial_number = response.data.serial_number;
      ctrl.system_version = response.data.system_version;
      ctrl.boot_version = response.data.boot_version;
    }, function(response) {
      // Second function handles error
      if (response.status < 500)
        ctrl.alert_import = response.data;
      else
        ctrl.alert_import = 'unknown';
    });
  };

  ctrl.firmware_upgrade = function() {
      $http({
        method: 'PUT',
        url: '/api/system/upgrade',
        data: { 'filename' : ctrl.upload_filename },
      }).then(function(response) {
        // First function handles success
        ctrl.alert_upgrade = 'success';
        ctrl.upgrading = false;
        ctrl.reboot();
      }, function(response) {
        // Second function handles error
        ctrl.alert_upgrade = response.data;
        ctrl.upgrading = false;
      });
  };

  ctrl.factory_reset = function() {
    ctrl.loading = true;
    $http.put('/api/system/factoryreset')
      .then(function(response) {
        // First function handles success
        ctrl.alert_factoryreset = true;
        ctrl.loading = false;
        ctrl.reboot();
      }, function(response) {
        // Second function handles error
        ctrl.loading = false;
      });
  };

  ctrl.reboot = function() {
    ctrl.alert_reboot = true;
    $http.put('/api/system/reboot')
      .then(function(response) {
        ctrl.counter = 30;
        $("#reboot_timer").modal();
        ctrl.countdown();
      });
  };

  ctrl.countdown = function() {
    $timeout(function() {
      if (ctrl.counter > 0) {
        ctrl.counter--;
        ctrl.countdown();
      } else {
        location.reload(true);
      }
    }, 1000);
  };

  ctrl.get();

}]);
